@@ -35,50 +35,8 @@
 #include "sysc/kernel/sc_cmnhdr.h"
 #include "sysc/communication/sc_mutex_if.h"
 
-#if defined(WIN32) || defined(_WIN32)
-
-#define SC_MTX_TYPE_ CRITICAL_SECTION
-
-#define SC_MTX_INIT_( Mutex ) \
-    InitializeCriticalSection( &(Mutex) )
-#define SC_MTX_LOCK_( Mutex ) \
-    EnterCriticalSection( &(Mutex) )
-#define SC_MTX_UNLOCK_( Mutex ) \
-    LeaveCriticalSection( &(Mutex) )
-#define SC_MTX_TRYLOCK_( Mutex ) \
-    ( TryEnterCriticalSection( &(Mutex) ) != 0 )
-#define SC_MTX_DESTROY_( Mutex ) \
-    DeleteCriticalSection( &(Mutex) )
-
-#else // use pthread mutex
-
-#include <pthread.h>
-#define SC_MTX_TYPE_ pthread_mutex_t
-
-#if defined(__hpux)
-#  define SC_PTHREAD_NULL_ cma_c_null
-#else // !defined(__hpux)
-#  define SC_PTHREAD_NULL_ NULL
-#endif
-
-#define SC_MTX_INIT_( Mutex ) \
-    pthread_mutex_init( &(Mutex), SC_PTHREAD_NULL_ )
-#define SC_MTX_LOCK_( Mutex ) \
-    pthread_mutex_lock( &(Mutex) )
-#define SC_MTX_UNLOCK_( Mutex ) \
-    pthread_mutex_unlock( &(Mutex) )
-
-#ifdef _XOPEN_SOURCE
-#   define SC_MTX_TRYLOCK_( Mutex ) \
-       ( pthread_mutex_trylock( &(Mutex) ) == 0 )
-#else // no try_lock available
-#   define SC_MTX_TRYLOCK_( Mutex ) \
-       ( false ) 
-#endif
-
-#define SC_MTX_DESTROY_( Mutex ) \
-    pthread_mutex_destroy( &(Mutex) )
-
+#if !defined(WIN32) && !defined(_WIN32) // use pthread mutex
+# include <pthread.h>
 #endif
 
 namespace sc_core {
@@ -89,33 +47,66 @@ namespace sc_core {
 //   The sc_host_mutex class, wrapping an OS mutex on the simulation host
 // ----------------------------------------------------------------------------
 
-class sc_host_mutex : public sc_mutex_if
+class SC_API sc_host_mutex : public sc_mutex_if
 {
-    typedef SC_MTX_TYPE_ underlying_type;
+#if defined(WIN32) || defined(_WIN32) // use CRITICAL_SECTION on Windows
+
+    typedef CRITICAL_SECTION underlying_type;
+
+    void do_init()    { InitializeCriticalSection( &m_mtx ); }
+    void do_lock()    { EnterCriticalSection( &m_mtx ); }
+    bool do_trylock() { return ( TryEnterCriticalSection( &m_mtx ) != 0 ); }
+    void do_unlock()  { LeaveCriticalSection( &m_mtx ); }
+    void do_destroy() { DeleteCriticalSection( &m_mtx ); }
+
+#else // use pthread mutex
+
+# if defined(__hpux)
+#   define SC_PTHREAD_NULL_ cma_c_null
+# else // !defined(__hpux)
+#   define SC_PTHREAD_NULL_ NULL
+# endif
+
+    typedef pthread_mutex_t underlying_type;
+
+    void do_init()    { pthread_mutex_init( &m_mtx, SC_PTHREAD_NULL_ ); }
+    void do_lock()    { pthread_mutex_lock( &m_mtx ); }
+# ifdef _XOPEN_SOURCE
+    bool do_trylock() { return ( pthread_mutex_trylock( &m_mtx ) == 0 ); }
+# else
+    bool do_trylock() { return /* not supported */ false; }
+# endif
+    void do_unlock()  { pthread_mutex_unlock( &m_mtx ); }
+    void do_destroy() { pthread_mutex_destroy( &m_mtx ); }
+
+# undef SC_PTHREAD_NULL_
+
+#endif // platform-specific implementation
+
 public:
 
     // constructors and destructor
 
-    sc_host_mutex()
-	{ SC_MTX_INIT_(m_mtx); }
+    sc_host_mutex() : m_mtx()
+      { do_init(); }
     virtual ~sc_host_mutex()
-	{ SC_MTX_DESTROY_(m_mtx); }
+      { do_destroy(); }
 
 
     // interface methods
 
     // blocks until mutex could be locked
     virtual int lock()
-	{ SC_MTX_LOCK_(m_mtx); return 0; }
+      { do_lock(); return 0; }
 
     // returns -1 if mutex could not be locked
     virtual int trylock()
-	{ return SC_MTX_TRYLOCK_(m_mtx) ? 0 : -1; }
+      { return do_trylock() ? 0 : -1; }
 
     // should return -1 if mutex was not locked by caller,
     // but is not yet able to check this
     virtual int unlock()
-	{ SC_MTX_UNLOCK_(m_mtx); return 0; }
+      { do_unlock(); return 0; }
 
 private:
     underlying_type m_mtx;
@@ -123,14 +114,6 @@ private:
 
 } // namespace sc_core
 
-#undef SC_MTX_TYPE_
-#undef SC_MTX_INIT_
-#undef SC_MTX_DESTROY_
-#undef SC_MTX_LOCK_
-#undef SC_MTX_TRYLOCK_
-#undef SC_MTX_UNLOCK_
-#undef SC_PTHREAD_NULL_
-
 /*****************************************************************************
 
   MODIFICATION LOG - modifiers, enter your name, affiliation, date and
