@@ -16,11 +16,10 @@
   permissions and limitations under the License.
 
  *****************************************************************************/
-#ifndef __MULTI_PASSTHROUGH_TARGET_SOCKET_H__
-#define __MULTI_PASSTHROUGH_TARGET_SOCKET_H__
+#ifndef TLM_UTILS_MULTI_PASSTHROUGH_TARGET_SOCKET_H_INCLUDED_
+#define TLM_UTILS_MULTI_PASSTHROUGH_TARGET_SOCKET_H_INCLUDED_
 
 #include "tlm_utils/multi_socket_bases.h"
-#include <sstream>
 
 namespace tlm_utils {
 
@@ -38,19 +37,11 @@ index of this socket the calling initiator is connected.
 template <typename MODULE,
           unsigned int BUSWIDTH = 32,
           typename TYPES = tlm::tlm_base_protocol_types,
-          unsigned int N=0
-#if !(defined SYSTEMC_VERSION & SYSTEMC_VERSION <= 20050714)
-          ,sc_core::sc_port_policy POL = sc_core::SC_ONE_OR_MORE_BOUND
-#endif
-          >
-class multi_passthrough_target_socket: public multi_target_base< BUSWIDTH,
-                                                        TYPES,
-                                                        N
-#if !(defined SYSTEMC_VERSION & SYSTEMC_VERSION <= 20050714)
-                                                        ,POL
-#endif
-                                                        >
-                              , public multi_to_multi_bind_base<TYPES>
+          unsigned int N=0,
+          sc_core::sc_port_policy POL = sc_core::SC_ONE_OR_MORE_BOUND>
+class multi_passthrough_target_socket
+  : public multi_target_base< BUSWIDTH, TYPES, N, POL>
+  , public multi_to_multi_bind_base<TYPES>
 {
 
 public:
@@ -67,27 +58,15 @@ public:
   typedef unsigned int (MODULE::*dbg_cb)(int, transaction_type& txn);
   typedef bool (MODULE::*dmi_cb)(int, transaction_type& txn, tlm::tlm_dmi& dmi);
 
-  typedef multi_target_base<BUSWIDTH,
-                        TYPES,
-                        N
-#if !(defined SYSTEMC_VERSION & SYSTEMC_VERSION <= 20050714)
-                        ,POL
-#endif
-                        > base_type;
+  typedef multi_target_base<BUSWIDTH, TYPES, N, POL> base_type;
 
   typedef typename base_type::base_initiator_socket_type base_initiator_socket_type;
 
-  //CTOR
-  multi_passthrough_target_socket()
-      : base_type(sc_core::sc_gen_unique_name("multi_passthrough_target_socket"))
-      , m_hierarch_bind(0)
-      , m_eoe_disabled(false)
-      , m_export_callback_created(false)
-  {
-  }
+  static const char* default_name()
+    { return sc_core::sc_gen_unique_name("multi_passthrough_target_socket"); }
 
   //CTOR
-  multi_passthrough_target_socket(const char* name)
+  explicit multi_passthrough_target_socket(const char* name = default_name())
       : base_type(name)
       , m_hierarch_bind(0)
       , m_eoe_disabled(false)
@@ -100,19 +79,6 @@ public:
     for (unsigned int i=0; i<m_binders.size(); i++) delete m_binders[i];
   }
 
-  //simple helpers for warnings an errors to shorten in code notation
-  void display_warning(const std::string& text) const {
-    std::stringstream s;
-    s<<"WARNING in instance "<<base_type::name()<<": "<<text;
-    SC_REPORT_WARNING("/OSCI_TLM-2/multi_socket", s.str().c_str());
-  }
-
-  void display_error(const std::string& text) const {
-    std::stringstream s;
-    s<<"ERROR in instance "<<base_type::name()<<": "<<text;
-    SC_REPORT_ERROR("/OSCI_TLM-2/multi_socket", s.str().c_str());
-  }
-
   void check_export_binding()
   {
     //if our export hasn't been bound yet (due to a hierarch binding)
@@ -129,7 +95,7 @@ public:
 
       if (m_binders.size() == 0)
       {
-        binder = new callback_binder_fw<TYPES>(m_binders.size());
+        binder = new callback_binder_fw<TYPES>(this, m_binders.size());
         m_binders.push_back(binder);
         m_export_callback_created = true;
       }
@@ -149,7 +115,7 @@ public:
     check_export_binding();
 
     //warn if there already is a callback
-    if (!m_nb_f.empty()){
+    if (m_nb_f.is_valid()){
       display_warning("NBTransport_bw callback already registered.");
       return;
     }
@@ -165,7 +131,7 @@ public:
     check_export_binding();
 
     //warn if there already is a callback
-    if (!m_b_f.empty()){
+    if (m_b_f.is_valid()){
       display_warning("BTransport callback already registered.");
       return;
     }
@@ -181,7 +147,7 @@ public:
     check_export_binding();
 
     //warn if there already is a callback
-    if (!m_dbg_f.empty()){
+    if (m_dbg_f.is_valid()){
       display_warning("DebugTransport callback already registered.");
       return;
     }
@@ -197,7 +163,7 @@ public:
     check_export_binding();
 
     //warn if there already is a callback
-    if (!m_dmi_f.empty()){
+    if (m_dmi_f.is_valid()){
       display_warning("DMI callback already registered.");
       return;
     }
@@ -217,10 +183,12 @@ public:
     //error if this socket is already bound hierarchically
     if (m_hierarch_bind) display_error("Socket already bound hierarchically.");
 
-    if (!m_export_callback_created)
-      m_binders.push_back(new callback_binder_fw<TYPES>(m_binders.size()));
-    else
+    if (m_export_callback_created) {
+      // consume binder created from the callback registration
       m_export_callback_created = false;
+    } else {
+      m_binders.push_back(new callback_binder_fw<TYPES>(this, m_binders.size()));
+    }
 
     return *m_binders[m_binders.size()-1];
   }
@@ -228,7 +196,7 @@ public:
   // const overload not allowed for multi-sockets
   virtual const tlm::tlm_fw_transport_if<TYPES>& get_base_interface() const
   {
-    display_error("'get_base_interface()' const not allowed for multi-sockets.");
+    display_error("'get_base_interface() const' not allowed for multi-sockets.");
     return base_type::get_base_interface();
   }
 
@@ -254,7 +222,12 @@ public:
     std::vector<callback_binder_fw<TYPES>* >& binders=get_hierarch_bind()->get_binders();
     std::map<unsigned int, tlm::tlm_bw_transport_if<TYPES>*>&  multi_binds=get_hierarch_bind()->get_multi_binds();
 
-    //iterate over all binders
+    // complete binding only if there has been a real bind
+    bool unbound = (binders.size() == 1 && m_export_callback_created);
+    // no call to get_base_interface has consumed the export - ignore
+    if (unbound) return;
+
+    // iterate over all binders
     for (unsigned int i=0; i<binders.size(); i++) {
       binders[i]->set_callbacks(m_nb_f, m_b_f, m_dmi_f, m_dbg_f); //set the callbacks for the binder
       if (multi_binds.find(i)!=multi_binds.end()) //check if this connection is multi-multi
@@ -302,6 +275,9 @@ public:
   unsigned int size(){return get_hierarch_bind()->get_binders().size();}
 
 protected:
+  using base_type::display_warning;
+  using base_type::display_error;
+
   //implementation of base class interface
   base_type* get_hierarch_bind(){if (m_hierarch_bind) return m_hierarch_bind->get_hierarch_bind(); else return this;}
   std::map<unsigned int, tlm::tlm_bw_transport_if<TYPES>*>&  get_multi_binds(){return m_multi_binds;}
@@ -323,8 +299,8 @@ protected:
   std::vector<callback_binder_fw<TYPES>*> m_binders;
 
   base_type*  m_hierarch_bind; //pointer to hierarchical bound multi port
-  bool m_eoe_disabled; //bool that diables callback bindings at end of elaboration
-  bool m_export_callback_created; // bool to indicate that a callback has already been created for export binding
+  bool m_eoe_disabled; //bool that disables callback bindings at end of elaboration
+  bool m_export_callback_created; //bool that indicates that a binder has been created from a callback registration
 
   //callbacks as functors
   // (allows to pass the callback to another socket that does not know the type of the module that owns
@@ -335,6 +311,18 @@ protected:
   typename callback_binder_fw<TYPES>::dmi_func_type   m_dmi_f;
 };
 
-}
+template <typename MODULE,
+          unsigned int BUSWIDTH = 32,
+          typename TYPES = tlm::tlm_base_protocol_types,
+          unsigned int N=0>
+class multi_passthrough_target_socket_optional
+  : public multi_passthrough_target_socket<MODULE,BUSWIDTH,TYPES,N,sc_core::SC_ZERO_OR_MORE_BOUND>
+{
+  typedef multi_passthrough_target_socket<MODULE,BUSWIDTH,TYPES,N,sc_core::SC_ZERO_OR_MORE_BOUND> socket_b;
+public:
+  multi_passthrough_target_socket_optional() : socket_b() {}
+  explicit multi_passthrough_target_socket_optional(const char* name) : socket_b(name) {}
+};
 
-#endif
+} // namespace tlm_utils
+#endif // TLM_UTILS_MULTI_PASSTHROUGH_TARGET_SOCKET_H_INCLUDED_
